"use client";

import React, { useState } from "react";
import Link from "next/link";
import { useForm, useFieldArray } from "react-hook-form";
import { Button } from "@/components/ui/button";
import {
  Form,
  FormItem,
  FormLabel,
  FormDescription,
  FormMessage,
} from "@/components/ui/form";

// Sample Indian lawyers/orgs
const LAWYERS = [
  { label: "Adv. Rohan Deshmukh (Mumbai)", value: "rohan-deshmukh" },
  { label: "Adv. Priya Nair (Delhi)", value: "priya-nair" },
  { label: "People's Justice Foundation", value: "peoples-justice" },
  { label: "Adv. Anil Kumar Choudhary (Kolkata)", value: "anil-choudhary" },
  { label: "Society for Legal Aid Trust", value: "legal-aid-society" },
];

function downloadMarkdown(filename: string, text: string) {
  const blob = new Blob([text], { type: "text/markdown" });
  const elem = document.createElement("a");
  elem.href = URL.createObjectURL(blob);
  elem.download = filename;
  elem.click();
}

// --- PDF rendering utilities using jsPDF ---
// Optionally install: npm i jspdf
// And "import jsPDF from 'jspdf';"
let jsPDF: any = null;
if (typeof window !== "undefined") {
  // @ts-ignore
  import("jspdf").then((mod) => (jsPDF = mod.default));
}

function createMarkdownFromForm(data: any, fileList: File[] = []) {
  const lawyer = LAWYERS.find((l) => l.value === data.lawyer)?.label ?? "";

  const docsSection =
    fileList.length === 0
      ? "_No files uploaded_"
      : fileList.map((f) => `- ${f.name}`).join("\n");

  const str = `# Public Interest Litigation (Draft)

**Filed by:** ${lawyer}

---

## 1. Aggrieved Parties

${data.aggrieved
  .map(
    (p: any, idx: number) =>
      `${idx + 1}. **${p.name}**\n   - Address: ${p.address}`,
  )
  .join("\n")}

---

## 2. Respondent Government Agencies

${data.government
  .map(
    (a: any, idx: number) =>
      `${idx + 1}. **${a.name}**\n   - Address: ${a.address}`,
  )
  .join("\n")}

---

## 3. Facts Leading To Violation of Fundamental Rights

${data.facts}

- **Period:** ${data.dates}
- **Prayers/Relief Sought:**
${data.prayers}

${data.precedence ? `**Precedents Cited:** ${data.precedence}\n` : ""}
${data.comments ? `**Extra Comments:** ${data.comments}\n` : ""}

---

## 4. Documents Submitted

${docsSection}
`;
  return str;
}

async function createAndDownloadPDF(markdown: string, filename: string) {
  if (!jsPDF) {
    alert(
      "PDF library is still loading. Please try again in a second, or try refreshing.",
    );
    return;
  }
  // Preprocess markdown: convert to text with basic formatting
  const mdToText = (md: string) =>
    md
      .replace(/^#+\s?/gm, "") // strip markdown headers
      .replace(/\*\*(.+?)\*\*/g, "$1") // bold
      .replace(/^- /gm, "• ")
      .replace(/_{1,2}(.+?)_{1,2}/g, "$1")
      .replace(/\r?\n/g, "\n");

  const text = mdToText(markdown);
  const doc = new jsPDF();
  const lines = doc.splitTextToSize(text, 180);
  doc.setFont("helvetica", "normal");
  doc.setFontSize(11);
  let y = 18;
  doc.text(filename.replace(/\.pdf$/i, ""), 10, 10, { maxWidth: 180 });
  lines.forEach((line: string) => {
    if (y > 285) {
      doc.addPage();
      y = 10;
    }
    doc.text(line, 10, y);
    y += 7;
  });
  doc.save(filename);
}

export default function FilePILPage() {
  const [output, setOutput] = useState<string | null>(null);
  const [uploadedFiles, setUploadedFiles] = useState<File[]>([]);

  const form = useForm({
    defaultValues: {
      lawyer: "",
      aggrieved: [{ name: "", address: "" }],
      government: [{ name: "", address: "" }],
      facts: "",
      dates: "",
      prayers: "",
      comments: "",
      precedence: "",
    },
  });

  const { control, register, handleSubmit, reset, formState } = form;
  const {
    fields: aggrievedFields,
    append: appendAggrieved,
    remove: removeAggrieved,
  } = useFieldArray({ control, name: "aggrieved" });
  const {
    fields: govFields,
    append: appendGovernment,
    remove: removeGovernment,
  } = useFieldArray({ control, name: "government" });

  const onSubmit = (data: any) => {
    const md = createMarkdownFromForm(data, uploadedFiles);
    setOutput(md);
    downloadMarkdown("PIL-draft.md", md);
  };

  const onRenderPdf = () => {
    if (!output) return;
    createAndDownloadPDF(output, "PIL-draft.pdf");
  };

  const handleFileChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    if (e.target.files) {
      setUploadedFiles(Array.from(e.target.files));
    }
  };

  return (
    <div className="min-h-screen flex flex-col font-sans antialiased">
      <header className="py-4 px-8 border-b bg-background sticky top-0 z-10">
        <div className="flex items-center justify-between max-w-2xl mx-auto">
          <span className="font-bold text-lg text-primary">🧑🏽‍⚖️ PIL Filing</span>
          <Link
            href="/"
            className="text-primary-foreground bg-primary px-3 py-1.5 rounded hover:opacity-90 text-sm"
          >
            ← Home
          </Link>
        </div>
      </header>

      <main className="flex-1 flex justify-center items-start py-6 px-2 bg-muted/10">
        <div className="w-full max-w-2xl bg-background rounded shadow-sm px-6 py-8 border">
          <h1 className="text-2xl font-bold mb-4 text-center">
            Draft & Submit PIL
          </h1>
          <Form {...form}>
            <form
              className="space-y-6"
              onSubmit={handleSubmit(onSubmit)}
              autoComplete="off"
            >
              {/* 1. Lawyer Selection */}
              <FormItem>
                <FormLabel>
                  Choose Public Interest Lawyer/Organization
                </FormLabel>
                <select
                  className="w-full border rounded px-3 py-2 bg-background"
                  {...register("lawyer", { required: "Choose a lawyer" })}
                  defaultValue=""
                >
                  <option value="" disabled>
                    Select a Lawyer/Org
                  </option>
                  {LAWYERS.map((law) => (
                    <option key={law.value} value={law.value}>
                      {law.label}
                    </option>
                  ))}
                </select>
                {formState.errors.lawyer && (
                  <FormMessage>
                    {formState.errors.lawyer.message as any}
                  </FormMessage>
                )}
              </FormItem>

              {/* 2. Files Uploader */}
              <FormItem>
                <FormLabel>
                  Upload Attachments (as a single or multiple files)
                </FormLabel>
                <input
                  type="file"
                  multiple
                  onChange={handleFileChange}
                  className="block text-sm mt-1"
                  accept="application/pdf,image/*"
                />
                <FormDescription className="text-[12px]">
                  Upload any documents, images, or supporting materials. (Files
                  are NOT uploaded to any server.)
                </FormDescription>
                {uploadedFiles.length > 0 && (
                  <ul className="text-xs text-muted-foreground mt-1">
                    {uploadedFiles.map((file) => (
                      <li key={file.name}>{file.name}</li>
                    ))}
                  </ul>
                )}
              </FormItem>

              {/* 3. Aggrieved Parties */}
              <fieldset className="border rounded px-3 py-2">
                <legend className="text-sm font-bold mb-2 px-1">
                  Aggrieved Parties{" "}
                  <span className="font-normal text-muted-foreground">
                    (Applicant[s])
                  </span>
                </legend>
                {aggrievedFields.map((item, idx) => (
                  <div key={item.id} className="flex gap-2 mb-2">
                    <input
                      type="text"
                      placeholder="Full Name"
                      {...register(`aggrieved.${idx}.name`, {
                        required: true,
                      })}
                      className="flex-1 px-2 py-1 rounded border"
                    />
                    <input
                      type="text"
                      placeholder="Address"
                      {...register(`aggrieved.${idx}.address`, {
                        required: true,
                      })}
                      className="flex-1 px-2 py-1 rounded border"
                    />
                    <Button
                      type="button"
                      variant="ghost"
                      size="icon"
                      onClick={() => removeAggrieved(idx)}
                      disabled={aggrievedFields.length === 1}
                    >
                      ×
                    </Button>
                  </div>
                ))}
                <Button
                  type="button"
                  size="sm"
                  variant="secondary"
                  onClick={() => appendAggrieved({ name: "", address: "" })}
                  className="mt-1"
                >
                  + Add Party
                </Button>
              </fieldset>

              {/* 4. Respondent Agencies */}
              <fieldset className="border rounded px-3 py-2">
                <legend className="text-sm font-bold mb-2 px-1">
                  Govt. Agencies/Depts (Respondents)
                </legend>
                {govFields.map((item, idx) => (
                  <div key={item.id} className="flex gap-2 mb-2">
                    <input
                      type="text"
                      placeholder="Agency Name"
                      {...register(`government.${idx}.name`, {
                        required: true,
                      })}
                      className="flex-1 px-2 py-1 rounded border"
                    />
                    <input
                      type="text"
                      placeholder="Address"
                      {...register(`government.${idx}.address`, {
                        required: true,
                      })}
                      className="flex-1 px-2 py-1 rounded border"
                    />
                    <Button
                      type="button"
                      variant="ghost"
                      size="icon"
                      onClick={() => removeGovernment(idx)}
                      disabled={govFields.length === 1}
                    >
                      ×
                    </Button>
                  </div>
                ))}
                <Button
                  type="button"
                  size="sm"
                  variant="secondary"
                  onClick={() => appendGovernment({ name: "", address: "" })}
                  className="mt-1"
                >
                  + Add Agency
                </Button>
              </fieldset>

              {/* 5. Facts */}
              <FormItem>
                <FormLabel>
                  Facts Leading to Violation of Fundamental Rights
                </FormLabel>
                <textarea
                  {...register("facts", { required: true })}
                  rows={3}
                  placeholder="Describe the situation, incidents, background..."
                  className="w-full border rounded px-3 py-2"
                />
                <FormMessage />
              </FormItem>

              {/* 6. Dates */}
              <FormItem>
                <FormLabel>Relevant Dates (of Grievance)</FormLabel>
                <input
                  type="text"
                  placeholder="e.g. Feb 2023 - May 2024"
                  {...register("dates", { required: true })}
                  className="w-full border rounded px-3 py-2"
                />
                <FormMessage />
              </FormItem>

              {/* 7. Prayers/Relief */}
              <FormItem>
                <FormLabel>Prayers / Relief Sought</FormLabel>
                <textarea
                  {...register("prayers", { required: true })}
                  rows={2}
                  placeholder="State what you want the court to grant."
                  className="w-full border rounded px-3 py-2"
                />
                <FormMessage />
              </FormItem>

              {/* 8. Extra/precedent */}
              <FormItem>
                <FormLabel>Precedent Cases (if any)</FormLabel>
                <input
                  type="text"
                  placeholder="E.g. Vishaka v. State of Rajasthan, AIR 1997 SC 3011"
                  {...register("precedence")}
                  className="w-full border rounded px-3 py-2"
                />
              </FormItem>
              <FormItem>
                <FormLabel>Additional Comments</FormLabel>
                <textarea
                  {...register("comments")}
                  rows={2}
                  placeholder="Any other relevant comment"
                  className="w-full border rounded px-3 py-2"
                />
              </FormItem>

              <div className="flex items-center justify-between mt-4 gap-2">
                <Button
                  type="button"
                  variant="ghost"
                  onClick={() => {
                    reset();
                    setOutput(null);
                    setUploadedFiles([]);
                  }}
                >
                  Reset Form
                </Button>
                <div className="flex gap-2">
                  <Button
                    type="button"
                    size="lg"
                    onClick={onRenderPdf}
                    disabled={!output}
                    variant="secondary"
                  >
                    Render as PDF
                  </Button>
                  <Button type="submit" size="lg">
                    Generate & Download PIL Markdown
                  </Button>
                </div>
              </div>
            </form>
          </Form>
          {output && (
            <div className="mt-8 border bg-muted rounded px-4 py-4 overflow-x-auto text-xs max-h-[320px]">
              <div className="font-bold mb-2 text-primary">
                Your Draft (preview):
              </div>
              <pre>{output}</pre>
            </div>
          )}
        </div>
      </main>
    </div>
  );
}
